<?php
/*
Plugin Name: Affiliate Link Manager
Description: Cloak and manage affiliate links with custom slugs without external APIs.
Version: 1.0
Author: Kirtish Vyas
*/

if (!defined('ABSPATH')) exit;

class CustomAffiliateLinkManager {
    private $post_type = 'calm_affiliate_link';

    public function __construct() {
        add_action('init', [$this, 'register_post_type']);
        add_action('add_meta_boxes', [$this, 'add_meta_boxes']);
        add_action('save_post', [$this, 'save_meta_box_data']);
        add_action('init', [$this, 'handle_redirection'], 1);
        add_filter('manage_' . $this->post_type . '_posts_columns', [$this, 'custom_columns']);
        add_action('manage_' . $this->post_type . '_posts_custom_column', [$this, 'custom_column_content'], 10, 2);
        add_action('admin_enqueue_scripts', [$this, 'admin_scripts']);
    }

    public function register_post_type() {
        register_post_type($this->post_type, [
            'labels' => [
                'name' => 'Affiliate Links',
                'singular_name' => 'Affiliate Link'
            ],
            'public' => false,
            'show_ui' => true,
            'supports' => ['title'],
            'menu_position' => 25,
            'menu_icon' => 'dashicons-admin-links'
        ]);
    }

    public function add_meta_boxes() {
        add_meta_box('calm_meta_box', 'Affiliate Link Details', [$this, 'render_meta_box'], $this->post_type);
    }

    public function render_meta_box($post) {
        wp_nonce_field('calm_save_meta_box', 'calm_meta_box_nonce');
        $target_url = get_post_meta($post->ID, '_calm_target_url', true);
        $custom_slug = get_post_meta($post->ID, '_calm_custom_slug', true);
        $redirect_type = get_post_meta($post->ID, '_calm_redirect_type', true);
        $notes = get_post_meta($post->ID, '_calm_notes', true);
        $clicks = get_post_meta($post->ID, '_calm_clicks', true);
        $pretty_link = home_url('/') . $custom_slug;
        ?>
        <p><strong>Target URL:</strong><br>
            <input type="url" name="calm_target_url" value="<?php echo esc_attr($target_url); ?>" class="widefat"></p>
        <p><strong>Custom Slug:</strong><br>
            <input type="text" name="calm_custom_slug" value="<?php echo esc_attr($custom_slug); ?>" class="widefat"></p>
        <p><strong>Redirect Type:</strong><br>
            <select name="calm_redirect_type" class="widefat">
                <option value="301" <?php selected($redirect_type, '301'); ?>>301 Moved Permanently</option>
                <option value="302" <?php selected($redirect_type, '302'); ?>>302 Found</option>
                <option value="307" <?php selected($redirect_type, '307'); ?>>307 Temporary Redirect</option>
            </select></p>
        <p><strong>Notes:</strong><br>
            <textarea name="calm_notes" class="widefat"><?php echo esc_textarea($notes); ?></textarea></p>
        <p><strong>Pretty Link:</strong><br>
            <input type="text" id="calm_pretty_link" value="<?php echo esc_url($pretty_link); ?>" readonly class="widefat">
            <button type="button" class="button" onclick="navigator.clipboard.writeText('<?php echo esc_url($pretty_link); ?>')">Copy</button></p>
        <p><strong>Click Count:</strong> <?php echo intval($clicks); ?></p>
        <?php
    }

    public function save_meta_box_data($post_id) {
        if (!isset($_POST['calm_meta_box_nonce']) || !wp_verify_nonce($_POST['calm_meta_box_nonce'], 'calm_save_meta_box')) {
            return;
        }
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return;
        if (!current_user_can('edit_post', $post_id)) return;

        update_post_meta($post_id, '_calm_target_url', esc_url_raw($_POST['calm_target_url']));
        update_post_meta($post_id, '_calm_custom_slug', sanitize_title_with_dashes($_POST['calm_custom_slug']));
        update_post_meta($post_id, '_calm_redirect_type', sanitize_text_field($_POST['calm_redirect_type']));
        update_post_meta($post_id, '_calm_notes', sanitize_textarea_field($_POST['calm_notes']));
    }

    public function handle_redirection() {
        if (is_admin()) return;

        $request_uri = trim(parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH), '/');
        $args = [
            'post_type' => $this->post_type,
            'posts_per_page' => -1
        ];
        $links = get_posts($args);

        foreach ($links as $link) {
            $slug = get_post_meta($link->ID, '_calm_custom_slug', true);
            if ($slug === $request_uri) {
                $url = get_post_meta($link->ID, '_calm_target_url', true);
                $type = intval(get_post_meta($link->ID, '_calm_redirect_type', true));
                $clicks = intval(get_post_meta($link->ID, '_calm_clicks', true));
                update_post_meta($link->ID, '_calm_clicks', $clicks + 1);
                wp_redirect($url, $type);
                exit;
            }
        }
    }

    public function custom_columns($columns) {
        $columns['calm_pretty_link'] = 'Pretty Link';
        $columns['calm_clicks'] = 'Clicks';
        return $columns;
    }

    public function custom_column_content($column, $post_id) {
        if ($column === 'calm_pretty_link') {
            $slug = get_post_meta($post_id, '_calm_custom_slug', true);
            $url = home_url('/') . $slug;
            echo '<a href="' . esc_url($url) . '" target="_blank">' . esc_html($url) . '</a>';
        }
        if ($column === 'calm_clicks') {
            echo intval(get_post_meta($post_id, '_calm_clicks', true));
        }
    }

    public function admin_scripts($hook) {
        if ('edit.php' !== $hook && 'post.php' !== $hook && 'post-new.php' !== $hook) return;
        wp_enqueue_script('calm-copy-js', plugin_dir_url(__FILE__) . 'copy.js', [], false, true);
    }
}

new CustomAffiliateLinkManager();
